/** 
 * XMLSec library
 *
 * Keys
 *
 * See Copyright for the status of this software.
 * 
 * Author: Aleksey Sanin <aleksey@aleksey.com>
 */
#ifndef __XMLSEC_KEYS_H__
#define __XMLSEC_KEYS_H__    

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */ 

#include <libxml/tree.h>
#include <openssl/x509.h>

#include <xmlsec/xmlsec.h>
#include <xmlsec/transforms.h>

#ifndef XMLSEC_KEYPTR_DEFINED
typedef struct _xmlSecKey 		*xmlSecKeyPtr; /* defined in keys.h */
#define XMLSEC_KEYPTR_DEFINED 			1
#endif /* XMLSEC_KEY_DEFINED */

#ifndef XMLSEC_KEYID_DEFINED
typedef const struct _xmlSecKeyId	*xmlSecKeyId; /* defined in keys.h */
#define XMLSEC_KEYID_DEFINED
#endif /* XMLSEC_KEYID_DEFINED */

/** 
 * Key Origins
 */
#define xmlSecKeyOriginDefault			0
#define xmlSecKeyOriginKeyManager		1
#define xmlSecKeyOriginKeyName			2 /* useless w/o xmlSecKeyOriginKeyManager */
#define xmlSecKeyOriginKeyValue			4
#define xmlSecKeyOriginRetrievalDocument	8
#define xmlSecKeyOriginRetrievalRemote		16
#define xmlSecKeyOriginX509			32
#define xmlSecKeyOriginPGP			64
#define xmlSecKeyOriginAll			\
	    (xmlSecKeyOriginKeyManager | xmlSecKeyOriginKeyName | \
	     xmlSecKeyOriginKeyValue | xmlSecKeyOriginKeyValue | \
	     xmlSecKeyOriginRetrievalDocument | xmlSecKeyOriginRetrievalRemote | \
	     xmlSecKeyOriginX509 | xmlSecKeyOriginPGP)		

/** 
 * Key Types
 */
typedef enum _xmlSecKeyType {
    xmlSecKeyTypePublic = 0,
    xmlSecKeyTypePrivate,
    xmlSecKeyTypeAny
} xmlSecKeyType;

/**
 * inifinite number of retrievals (really big number :) )
 */
#define xmlSecKeyInifiteRetrivals		99999

typedef long				xmlSecKeyOrigin;
typedef struct _xmlSecKeysReadContext	*xmlSecKeysReadContextPtr;
    
typedef xmlSecKeyPtr	(*xmlSecKeyCreateMethod)	(xmlSecKeyId id);
typedef void		(*xmlSecKeyDestroyMethod)	(xmlSecKeyPtr key);
typedef int		(*xmlSecKeyGenerateMethod)	(xmlSecKeyPtr key,
							 void *context);
typedef int		(*xmlSecKeyReadXmlMethod)	(xmlSecKeyPtr key,
							 xmlNodePtr node);
typedef int		(*xmlSecKeyWriteXmlMethod)	(xmlSecKeyPtr key,
							 xmlSecKeyType type,
							 xmlNodePtr parent);


#define xmlSecKeyIdUnknown 			NULL

struct _xmlSecKeyId {
    /* xlmlSecKeyId data */
    const xmlChar 			*keyValueNodeName;
    const xmlChar			*keyValueNodeNs;
    
    /* xmlSecKeyId methods */
    xmlSecKeyGenerateMethod		generate;
    xmlSecKeyCreateMethod		create;
    xmlSecKeyDestroyMethod		destroy;
    xmlSecKeyReadXmlMethod		read;
    xmlSecKeyWriteXmlMethod		write;
};

struct _xmlSecKey {
    xmlSecKeyId				id;
    xmlSecKeyType			type;
    xmlChar				*name;
    xmlSecKeyOrigin			origin;
    
    /* x509 cert used to create the key (if any) */
    X509				*x509Cert;
    
    /* x509 certs that verifies the key (including the x509Cert!) */
    STACK_OF(X509) 			*x509Chain;
    
    /* key specific data */
    void				*keyData;
};


typedef xmlSecKeyPtr 	(*xmlSecKeyFindKeyCallback)	(void *context,
							 xmlSecTransformId id, 
							 xmlSecKeyType type,
						         const xmlChar *name);

typedef X509*		(*xmlSecX509FindCallback)	(void *context,
							 xmlChar *subjectName,
							 xmlChar *issuerName,
							 xmlChar *issuerSerial,
							 xmlChar *ski);
/**
 * xmlSecX509VerifyCallback:
 * @context: the application specific context
 * @cert: the cert to verify
 * @untrusted: untrusted ceritficates from the chain
 *
 * Returns 1 if the cert is trusted, 0 if it is not trusted
 * and -1 if an error occurs
 */
typedef int 		(*xmlSecX509VerifyCallback)	(void *context,
							 X509 *cert,
							 STACK_OF(X509) *untrusted);
/** 
 * xmlSecX509AddCRLCallback:
 * @context:
 * @crl:	the verified CRL to add
 *
 */
typedef int		(*xmlSecX509AddCRLCallback)	(void *context,
							 X509_CRL *crl);

typedef struct _xmlSecKeysReadContext {
    xmlSecKeyOrigin 		allowedOrigins;
    int 			maxRetrievals;
    
    /* key manager context */
    xmlSecKeyFindKeyCallback 	findKeyCallback;
    void 			*findKeyContext;   
     
    /* x509 context */
    xmlSecX509FindCallback	x509FindCallback;
    xmlSecX509VerifyCallback	x509VerifyCallback;
    xmlSecX509AddCRLCallback	x509AddCRLCallback;
    void			*x509Context;
} xmlSecKeysReadContext;

/** 
 * XML Sec Key
 */
void		xmlSecKeysInit				(void); 
 
 
#define xmlSecKeyIsValid(key) \
	((( key ) != NULL) && ((( key )->id) != NULL))
#define xmlSecKeyCheckId(key, keyId) \
 	(xmlSecKeyIsValid(( key )) && \
	((( key )->id) == ( keyId )))
#define xmlSecKeyCheckTransform(key, tr) \
 	(xmlSecKeyIsValid(( key )) && \
	((((const xmlSecKeyId) (( key )->id->transformId))) == ( tr )))
xmlSecKeyPtr	xmlSecKeyCreate				(xmlSecKeyId id,
							 xmlSecKeyOrigin origin);
void		xmlSecKeyDestroy			(xmlSecKeyPtr key,
							 int forceDestroy);
xmlSecKeyPtr	xmlSecKeyReadEvpKey			(EVP_PKEY *pKey, 
							 xmlSecTransformId id, 
							 xmlSecKeyOrigin origin);
xmlSecKeyPtr	xmlSecKeyGenerate			(xmlSecKeyId id,
							 void *context);
xmlSecKeyPtr	xmlSecKeyReadXml			(xmlSecKeyId id,
							 xmlNodePtr node);
int		xmlSecKeyWriteXml			(xmlSecKeyPtr key,
							 xmlSecKeyType type,
							 xmlNodePtr node); 							
void		xmlSecDebugKeyDump			(FILE *output,
							 xmlSecKeyPtr key);

/** 
 * XMLDsig:KeyInfo methods
 */ 
xmlSecKeyPtr	xmlSecKeyInfoRead			(const xmlNodePtr keyInfoNode,
							 xmlSecTransformId id,
							 xmlSecKeyType type,
							 const xmlSecKeysReadContext * ctx);
int		xmlSecKeyInfoWrite			(xmlNodePtr keyInfoNode,
							 const xmlSecKeyPtr key,
							 xmlSecKeyType type);

/** 
 *
 */							 
xmlSecKeyPtr 	xmlSecKeySearcCtxFindKey		(const xmlSecKeysReadContext *keysContext,
							 xmlSecTransformId id,
							 xmlSecKeyType type,
							 const xmlChar *name);
    
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __XMLSEC_KEYS_H__ */

