/** 
 * XMLSec library
 *
 * "XML Digital Signature" implementation
 *  http://www.w3.org/TR/xmldsig-core/
 *  http://www.w3.org/Signature/Overview.html
 * 
 * See Copyright for the status of this software.
 * 
 * Author: Aleksey Sanin <aleksey@aleksey.com>
 */
#ifndef __XMLSEC_XMLDSIG_H__
#define __XMLSEC_XMLDSIG_H__    

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */ 
#include <libxml/tree.h>
#include <libxml/parser.h> 

#include <xmlsec/xmlsec.h>
#include <xmlsec/keys.h>
#include <xmlsec/transforms.h>


typedef struct _xmlDSigCtx *xmlDSigCtxPtr; 
typedef struct _xmlDSigSignature *xmlDSigSignaturePtr;
typedef struct _xmlDSigReference *xmlDSigReferencePtr;

typedef enum _xmlDSigReferenceType {
    xmlDSigSignedInfoReference,
    xmlDSigManifestReference
} xmlDSigReferenceType;

/** 
 * DSig context
 */
typedef struct _xmlDSigCtx {
    /* keys search context */
    const xmlSecKeysReadContext	*keysReadContext;    
    
    /* flags */
    int				processManifests;/* if 0 then Manifests are
						  * not processed */
    int				storeSignatures; /* store the signed content 
						  * just (SignedInfo element)
						  * before applying signature */						  
    int				storeReferences; /* store the result of processing
						  * Reference node in SignedInfo
						  *just before digesting */
    int				storeManifests;	 /* store the result of processing
						  * Reference node in Manifest
						  * just before digesting 
						  * (ignored if processManifest is 0)
						  */
} xmlDSigCtx;

/**
 * DSig Signature
 */
typedef struct _xmlDSigSignature {
    xmlDSigCtxPtr		ctx;
    xmlNodePtr			self;
    int				sign;
    xmlSecTransformStatus	result;
    xmlDSigSignaturePtr		next;
    xmlDSigSignaturePtr		prev;    
    xmlSecTransformId		signMethod;
    xmlSecKeyPtr		signKey;
    xmlDSigReferencePtr		firstSignRef;	/* the link to the first reference */
    xmlDSigReferencePtr		lastSignRef;	/* the link to the last reference */    
    xmlDSigReferencePtr		firstManifestRef;
    xmlDSigReferencePtr		lastManifestRef;    	
    xmlBufferPtr		buffer;
} xmlDSigSignature;

/**
 * DSig Reference
 */
typedef struct _xmlDSigReference {
    xmlDSigCtxPtr		ctx;
    xmlNodePtr			self;
    xmlDSigReferenceType	refType;
    xmlSecTransformStatus	result;
    xmlDSigReferencePtr		next;
    xmlDSigReferencePtr		prev;
    xmlChar			*uri;
    xmlChar			*id;
    xmlChar			*type;
    xmlSecTransformId		digestMethod;
    xmlBufferPtr		buffer;
} xmlDSigReference; 


/**
 * DSig context methods
 */
xmlDSigCtxPtr		xmlDSigCtxCreate		(const xmlSecKeysReadContext *keysReadContext);
void 			xmlDSigCtxDestroy		(xmlDSigCtxPtr ctx);


/**
 * DSig generation/validation
 */
int			xmlDSigValidate			(xmlDSigCtxPtr ctx,
							 xmlNodePtr cur,
							 xmlDSigSignaturePtr *signature);
int			xmlDSigGenerate			(xmlDSigCtxPtr ctx,
							 xmlNodePtr cur,
							 xmlDSigSignaturePtr *signature);
/**
 * DSig results methods
 */
void			xmlDSigSignatureDestroyAll	(xmlDSigSignaturePtr signature);
void			xmlDSigDebugSignatureDumpAll	(FILE *output,
							 xmlDSigSignaturePtr signature);


/**
 * Creating DSig "on-the-fly"
 */			
xmlNodePtr		xmlDSigSignatureNodeCreate	(xmlDocPtr doc,
							 const xmlChar *id,
							 xmlSecTransformId c14nMethod,
							 void *c14nData,
							 xmlSecTransformId signMethod,
							 void *signData);
xmlNodePtr		xmlDSigReferenceNodeCreate	(xmlNodePtr signatureNode,
							 const xmlChar *id,
							 const xmlChar *uri,
							 const xmlChar *type,
							 xmlSecTransformId digestMethod,
							 void *digestData);
xmlNodePtr		xmlDSigTransformNodeCreate	(xmlNodePtr referenceNode,
							 xmlSecTransformId transformMethod,
							 void *transformData);
xmlNodePtr		xmlDSigKeyInfoNodeCreate	(xmlNodePtr signatureNode,
							 const xmlChar *id,
							 xmlSecKeyOrigin origin);
xmlNodePtr		xmlDSigObjectNodeCreate		(xmlNodePtr signatureNode,
							 const xmlChar *id);
xmlNodePtr		xmlDSigManifestNodeCreate	(xmlNodePtr signatureNode,
							 const xmlChar *id);
 				 
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __XMLSEC_XMLDSIG_H__ */

